@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 2018 Yubico AB. All rights reserved.
+ * Copyright (c) 2018-2023 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 /*
@@ -12,7 +13,7 @@
  * $ echo user name >> cred_param
  * $ dd if=/dev/urandom bs=1 count=32 | base64 >> cred_param
  * $ fido2-cred -M -i cred_param /dev/hidraw5 | fido2-cred -V -o cred
- */ 
+ */
 
 #include <fido.h>
 #include <stdio.h>
@@ -26,8 +27,8 @@ void
 usage(void)
 {
 	fprintf(stderr,
-"usage: fido2-cred -M [-dhqruv] [-i input_file] [-o output_file] device [type]\n"
-"       fido2-cred -V [-dhv] [-i input_file] [-o output_file] [type]\n"
+"usage: fido2-cred -M [-bdhqruvw] [-c cred_protect] [-i input_file] [-o output_file] device [type]\n"
+"       fido2-cred -V [-dhv] [-c cred_protect] [-i input_file] [-o output_file] [type]\n"
 	);
 
 	exit(1);
